@@ -1,7 +1,7 @@
 /*
  * cddb-slave.h: Header for CDDBSlave object
  *
- * Copyright (C) 2001 Ximian, Inc.
+ * Copyright (C) 2001-2002 Iain Holmes 
  *
  * Authors: Iain Holmes  <iain@ximian.com>
  */
@@ -27,6 +27,21 @@ extern "C" {
 typedef struct _CDDBSlave CDDBSlave;
 typedef struct _CDDBSlavePrivate CDDBSlavePrivate;
 typedef struct _CDDBSlaveClass CDDBSlaveClass;
+typedef struct _CDDBEntry {
+	char *realdiscid; /* The discid of the file and discid of the contents may not be the same */
+	char *discid;
+	int ntrks;
+	int revision;
+	int disc_length;
+
+	char **topline;
+	int *offsets;
+	int *lengths;
+
+	GList *comments;
+	GHashTable *fields;
+} CDDBEntry;
+
 
 struct _CDDBSlave {
 	BonoboObject parent;
@@ -42,9 +57,13 @@ struct _CDDBSlaveClass {
 
 GType cddb_slave_get_type ();
 CDDBSlave *cddb_slave_new (const char *server,
-			   int port);
+			   int port,
+			   const char *name,
+			   const char *hostname);
 CDDBSlave *cddb_slave_new_full (const char *server,
 				int port,
+				const char *name,
+				const char *hostname,
 				BonoboEventSource *event_source);
 BonoboEventSource *cddb_slave_get_event_source (CDDBSlave *cddb);
 
