@@ -108,7 +108,7 @@ GtkWidget  *slidernotebook;
 
 /* Menus */
 static GnomeUIInfo help_menu[] = {
-	GNOMEUIINFO_ITEM_STOCK(N_("Help"), NULL, help, GNOME_STOCK_PIXMAP_HELP),
+/*  	GNOMEUIINFO_ITEM_STOCK(N_("Help"), NULL, help, GNOME_STOCK_PIXMAP_HELP), */
 	GNOMEUIINFO_MENU_ABOUT_ITEM (about_cb, NULL),
 	GNOMEUIINFO_END
 };
@@ -132,6 +132,7 @@ static GnomeUIInfo main_menu[] = {
 /* End of menus */ 
 
 GList *devices;
+GHashTable *device_by_name = NULL;
 
 int mode=0, mode_norestore=0, mode_initonly=0, mode_nosave=0, num_mixers, mode_restore=0;
 #define M_NORESTORE	1
@@ -216,6 +217,7 @@ snd_mixer_callbacks_t read_cbs;
 static void rebuild_cb(void *data) {
 	fprintf(stderr, "gmix rebuild_cb()\n");
 }
+
 static void element_cb(void *data, int cmd, snd_mixer_eid_t *eid) {
 	device_info *device=(device_info *)data;
 	int err;
@@ -310,17 +312,19 @@ static void error_close_cb(void)
 
 void config_cb(GtkWidget *widget, gpointer data)
 {
-	prefs_make_window();
+	prefs_make_window (app);
 };
 
-int main(int argc, char *argv[]) 
+int
+main(int argc,
+     char *argv[]) 
 {
 	mode=0;
 
 	bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 	textdomain (GETTEXT_PACKAGE);
-	gnome_init_with_popt_table("gmix", VERSION, argc, argv, options,
+	gnome_init_with_popt_table("gnome-volume-control", VERSION, argc, argv, options,
 				   0, NULL);
 
 	if (g_file_exists (GNOME_ICONDIR"/gnome-volume.png"))
@@ -352,8 +356,6 @@ int main(int argc, char *argv[])
 		if (~mode & M_INITONLY) {
 
 			open_dialog();
-
-			put_gui_config();/* Do we need this*/
 		}
 
 		if (~mode & M_NOSAVE) {
@@ -517,7 +519,7 @@ device_info *open_device(int num)
 	if ((res==EINVAL) || (ver!=SOUND_VERSION)) {
 		if (res == EINVAL) {
 			fprintf(stderr, 
-				_("Warning: This version of gmix was compiled with\n"
+				_("Warning: This version of the Gnome Volume Control was compiled with\n"
 				"OSS version %d.%d.%d, and your system is running\n"
 				"a version prior to 3.6.0.\n"), 
 				SOUND_VERSION >> 16, 
@@ -525,7 +527,7 @@ device_info *open_device(int num)
 				SOUND_VERSION & 0xff);
 		} else {
 			fprintf(stderr, 
-				_("Warning: This version of gmix was compiled with\n"
+				_("Warning: This version of the Gnome Volume Control was compiled with\n"
 				"OSS version %d.%d.%d, and your system is running\n"
 				"version %d.%d.%d.\n"), 
 				SOUND_VERSION >> 16, 
@@ -538,6 +540,7 @@ device_info *open_device(int num)
 	/*
 	 * mixer-name
 	 */
+#ifndef __FreeBSD__
 	res=ioctl(new_device->fd, SOUND_MIXER_INFO, &new_device->info);
 	if (res!=0) {
 		g_free(new_device);
@@ -545,6 +548,9 @@ device_info *open_device(int num)
 	}
 	if(!isalpha(new_device->info.name[0]))
 		g_snprintf(new_device->info.name, 31, "Card %d", num+1);
+#else
+	g_snprintf(new_device->info.name, 31, "Card %d", num+1);
+#endif
 	/* 
 	 * several bitmasks describing the mixer
 	 */
@@ -673,19 +679,32 @@ GList *make_channels(device_info *device)
 	return channels;
 }
 
-void scan_devices(void)
+void
+scan_devices (void)
 {
 	int cnt;
 	device_info *new_device;
-	cnt=0; devices=NULL;
+	
+	cnt = 0;
+	devices = NULL;
+	device_by_name = g_hash_table_new (g_str_hash, g_str_equal);
+		
 	do {
-		new_device=open_device(cnt++);
+		new_device = open_device(cnt++);
 		if (new_device) {
-			new_device->channels=make_channels(new_device);
-			devices=g_list_append(devices, new_device);
+			new_device->channels = make_channels(new_device);
+			devices = g_list_append(devices, new_device);
+			g_hash_table_insert (device_by_name, new_device->info.name, new_device);
 		}
 	} while (new_device);
-	num_mixers=cnt-1;
+
+	num_mixers = cnt - 1;
+}
+
+device_info *
+device_from_name (const char *name)
+{
+	return g_hash_table_lookup (device_by_name, name);
 }
 
 #ifdef ALSA
@@ -980,50 +999,69 @@ void fill_in_device_guis(GtkWidget *notebook){
 	int i,j;
 
 	i=0;
+	if (num_mixers == 1) {
+		gtk_notebook_set_show_tabs (GTK_NOTEBOOK (notebook), FALSE);
+	}
+	
 	for (d=devices; d; d=d->next) {
 		device_info *di;
 		GList *table_focus_chain = NULL;
+
 		di=d->data;
 		j=0;
+		
 		for (c=((device_info *)d->data)->channels;c;c=c->next) {
 			j+=2;
 		}
 
-		/* 
-		 * David: changed 7 to 8 for table rows (06/04/1999)
-		 */
-		table = gtk_table_new (i*2, 8, FALSE);
+		table = gtk_table_new (i*2, 5, FALSE);
 		gtk_notebook_append_page (GTK_NOTEBOOK(notebook),
 					  table, 
 					  gtk_label_new(di->info.name));
 		gtk_table_set_row_spacings (GTK_TABLE (table), 0);
 		gtk_table_set_col_spacings (GTK_TABLE (table), 0);
 		gtk_container_border_width (GTK_CONTAINER (table), 0);
-		gtk_widget_show (table);
 
 		for (c= ((device_info *)d->data)->channels; c; c=c->next) {
 			GtkWidget *label, *mixer, *separator;
+			GtkWidget *hbox;
 			channel_info *ci;
 
-			ci=c->data;
+			ci = c->data;
 
-			if ((ci->pixmap) && (prefs.use_icons)) {
+			hbox = gtk_hbox_new (FALSE, 2);
+			gtk_widget_show (hbox);
+			gtk_table_attach (GTK_TABLE (table), hbox, i, i+1,
+					  0, 1, GTK_EXPAND | GTK_FILL,
+					  GTK_FILL, 0, 0);
+			if (ci->pixmap) {
 				spixmap = gtk_image_new_from_stock (ci->pixmap,
 								    GTK_ICON_SIZE_BUTTON);
-				gtk_table_attach (GTK_TABLE (table), spixmap, i, i+1, 1, 2, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
-				gtk_widget_show (spixmap);
+				gtk_box_pack_start (GTK_BOX (hbox), spixmap, FALSE, FALSE, 0);
+				if (prefs.show_icons == TRUE) {
+					gtk_widget_show (spixmap);
+				}
+				
+				ci->icon = spixmap;
+			} else {
+				ci->icon = NULL;
 			}
-			if (prefs.use_labels) {
-				label=gtk_label_new(ci->user_title);
-				gtk_misc_set_alignment (GTK_MISC(label), 0.1, 0.5);
-				gtk_table_attach (GTK_TABLE (table), label, i, i+1, 2, 3, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
-				gtk_widget_show(label);
+			
+			label = gtk_label_new (ci->user_title);
+			gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
+			gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_CENTER);
+			gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 0);
+			
+			if (prefs.show_labels == TRUE) {
+				gtk_widget_show (label);
 			}
+			ci->label = label;
 
 			mixer=make_slider_mixer(ci);
 			gtk_table_attach (GTK_TABLE (table), mixer, i, i+1,\
-					  3, 4, GTK_EXPAND | GTK_FILL,\
+					  1, 2, GTK_EXPAND | GTK_FILL,\
 					  GTK_EXPAND | GTK_FILL, 0, 0);
+			gtk_widget_set_size_request(mixer, -1, 100);
 			gtk_widget_show (mixer);
 			table_focus_chain = g_list_prepend (table_focus_chain, mixer);
 			
@@ -1031,10 +1069,11 @@ void fill_in_device_guis(GtkWidget *notebook){
 				/* lock-button, only useful for stereo */
 				AtkObject *accessible;
 				gchar *accessible_name;
-				ci->lock = gtk_check_button_new_with_label(_("Lock"));ci->lock = gtk_check_button_new_with_label(_("Lock"));			  
+
+				ci->lock = gtk_check_button_new_with_label(_("Lock"));
 				gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ci->lock), (ci->device->lock_bitmask & (1<<ci->channel))!=0);
 				gtk_signal_connect (GTK_OBJECT (ci->lock), "toggled", (GtkSignalFunc) lock_cb, (gpointer)ci);
-				gtk_table_attach (GTK_TABLE (table), ci->lock, i, i+1, 4, 5, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
+				gtk_table_attach (GTK_TABLE (table), ci->lock, i, i+1, 2, 3, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
 				gtk_widget_show (ci->lock);
 				table_focus_chain = g_list_prepend (table_focus_chain, ci->lock);
 				accessible_name = g_strdup_printf ("%s %s",
@@ -1043,6 +1082,23 @@ void fill_in_device_guis(GtkWidget *notebook){
 				atk_object_set_name (accessible, accessible_name);
 				g_free (accessible_name);
 			}
+
+			if (ci->device->devmask & (1<<ci->channel)) {
+				AtkObject *accessible;
+				gchar *accessible_name;
+				ci->mute=gtk_check_button_new_with_label(_("Mute"));
+				gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ci->mute), (ci->device->mute_bitmask & (1<<ci->channel))!=0);
+				gtk_signal_connect (GTK_OBJECT (ci->mute), "toggled", (GtkSignalFunc) mute_cb, (gpointer)ci);
+				gtk_table_attach (GTK_TABLE (table), ci->mute, i, i+1, 4, 5, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
+				gtk_widget_show (ci->mute);
+				table_focus_chain = g_list_prepend (table_focus_chain, ci->mute);
+				accessible_name = g_strdup_printf ("%s %s",
+								   ci->user_title, _("Mute"));
+				accessible = gtk_widget_get_accessible (ci->mute);
+				atk_object_set_name (accessible, accessible_name);
+				g_free (accessible_name);
+			}
+
 			/*
 			 * recording sources
 			 */
@@ -1055,7 +1111,7 @@ void fill_in_device_guis(GtkWidget *notebook){
 				/* David: changed TOP_ATTACH to 7 and
 				 * BOTTOM_ATTACH to 8   06/04/1999
 				 */
-				gtk_table_attach (GTK_TABLE (table), ci->rec, i, i+1, 7, 8, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
+				gtk_table_attach (GTK_TABLE (table), ci->rec, i, i+1, 5, 6, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
 				gtk_widget_show (ci->rec);
 				table_focus_chain = g_list_prepend (table_focus_chain, ci->rec);
 				accessible_name = g_strdup_printf ("%s %s",
@@ -1072,27 +1128,13 @@ void fill_in_device_guis(GtkWidget *notebook){
 				ci->rec = NULL;
 			}
 	
-			if (ci->device->devmask & (1<<ci->channel)) {
-				AtkObject *accessible;
-				gchar *accessible_name;
-				ci->mute=gtk_check_button_new_with_label(_("Mute"));
-				gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ci->mute), (ci->device->mute_bitmask & (1<<ci->channel))!=0);
-				gtk_signal_connect (GTK_OBJECT (ci->mute), "toggled", (GtkSignalFunc) mute_cb, (gpointer)ci);
-				gtk_table_attach (GTK_TABLE (table), ci->mute, i, i+1, 6, 7, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
-				gtk_widget_show (ci->mute);
-				table_focus_chain = g_list_prepend (table_focus_chain, ci->mute);
-				accessible_name = g_strdup_printf ("%s %s",
-								   ci->user_title, _("Mute"));
-				accessible = gtk_widget_get_accessible (ci->mute);
-				atk_object_set_name (accessible, accessible_name);
-				g_free (accessible_name);
-			}
-
 			separator = gtk_vseparator_new ();
 			/* BOTTOM_ATTACH changed to 8 */
-			gtk_table_attach (GTK_TABLE (table), separator, i+1, i+2, 1, 8, GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
+			gtk_table_attach (GTK_TABLE (table), separator, i+1, i+2, 0, 6, GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
 			gtk_widget_show (separator);
 			i+=2; 
+
+			gtk_widget_show (table);
 		}
 	
 		/* Set the new focus chain for the table */
@@ -1418,7 +1460,7 @@ help(GtkWidget *widget,
      gpointer data)
 {
 	GnomeProgram *program;
-	GError *error;
+	GError *error = NULL;
 	gboolean ret;
 	
 	program = gnome_program_get ();
@@ -1429,22 +1471,6 @@ help(GtkWidget *widget,
 	}
 }
 
-void 
-help_cb(GtkWidget *widget, 
-	gpointer data)
-{
-	GnomeProgram *program;
-	GError *error;
-	gboolean ret;
-
-	program = gnome_program_get ();
-        ret = gnome_help_display("gmix-prefs.html", NULL, &error);
-	if (ret == FALSE) {
-		g_warning ("Error displaying document: %s", error->message);
-		g_error_free (error);
-	}
-}
-
 static void about_cb (GtkWidget *widget, gpointer data)
 {
 	static GtkWidget *about = NULL;
@@ -1461,7 +1487,7 @@ static void about_cb (GtkWidget *widget, gpointer data)
 	}
 	else
 	{
-		about = gnome_about_new ( _("GMIX - The Gnome Mixer"), VERSION,
+		about = gnome_about_new ( _("Gnome Volume Control"), VERSION,
 					  "(C) 1998 Jens Ch. Restemeier",
 					  _("This is a mixer for sound devices"),
 					  authors,
@@ -1473,3 +1499,53 @@ static void about_cb (GtkWidget *widget, gpointer data)
 		gtk_widget_show (about);
 	}
 }
+
+void
+gmix_change_icons (gboolean show)
+{
+	GList *p, *q;
+
+	for (p = devices; p; p = p->next) {
+		device_info *di = p->data;
+		
+		for (q = di->channels; q; q = q->next) {
+			channel_info *ci = q->data;
+			
+			if (ci->icon == NULL) {
+				continue;
+			}
+			
+			if (show == TRUE) {
+				gtk_widget_show (ci->icon);
+			} else {
+				gtk_widget_hide (ci->icon);
+			}
+		}
+	}
+}
+
+void
+gmix_change_labels (gboolean show)
+{
+	GList *p, *q;
+
+	for (p = devices; p; p = p->next) {
+		device_info *di = p->data;
+		
+		for (q = di->channels; q; q = q->next) {
+			channel_info *ci = q->data;
+
+			if (ci->label == NULL) {
+				continue;
+			}
+			
+			if (show == TRUE) {
+				gtk_widget_show (ci->label);
+			} else {
+				gtk_widget_hide (ci->label);
+			}
+		}
+	}
+}
+
+		
