@@ -16,10 +16,20 @@
 
 #include <libgnome/gnome-i18n.h>
 
+#include <bonobo/bonobo-exception.h>
+#include <bonobo/bonobo-control.h>
+#include <bonobo/bonobo-widget.h>
+
+#include <bonobo-activation/bonobo-activation.h>
+
 #include "cdrom.h"
 #include "display.h"
 #include "gnome-cd.h"
 
+#include "GNOME_Media_CDDBSlave2.h"
+
+static GNOME_Media_CDDBTrackEditor track_editor = CORBA_OBJECT_NIL;
+
 static void
 maybe_close_tray (GnomeCD *gcd)
 {
@@ -27,7 +37,7 @@ maybe_close_tray (GnomeCD *gcd)
 	GError *error;
 
 	if (gnome_cdrom_get_status (gcd->cdrom, &status, &error) == FALSE) {
-		g_warning ("%s: %s", __FUNCTION__, error->message);
+		g_warning ("%s: %s", G_GNUC_FUNCTION, error->message);
 		g_error_free (error);
 		
 		return;
@@ -35,7 +45,7 @@ maybe_close_tray (GnomeCD *gcd)
 
 	if (status->cd == GNOME_CDROM_STATUS_TRAY_OPEN) {
 		if (gnome_cdrom_close_tray (gcd->cdrom, &error) == FALSE) {
-			g_warning ("%s: %s", __FUNCTION__, error->message);
+			g_warning ("%s: %s", G_GNUC_FUNCTION, error->message);
 			g_error_free (error);
 		}
 	}
@@ -51,7 +61,7 @@ eject_cb (GtkButton *button,
 	GError *error;
 
 	if (gnome_cdrom_eject (gcd->cdrom, &error) == FALSE) {
-		g_warning ("%s: %s", __FUNCTION__, error->message);
+		g_warning ("%s: %s", G_GNUC_FUNCTION, error->message);
 		g_error_free (error);
 	}
 
@@ -79,7 +89,7 @@ play_cb (GtkButton *button,
 	GnomeCDRomMSF *endmsf;
 
 	if (gnome_cdrom_get_status (gcd->cdrom, &status, &error) == FALSE) {
-		g_warning ("%s: %s", __FUNCTION__, error->message);
+		g_warning ("%s: %s", G_GNUC_FUNCTION, error->message);
 		g_error_free (error);
 		return;
 
@@ -108,7 +118,7 @@ play_cb (GtkButton *button,
 		}
 		
 		if (gnome_cdrom_play (gcd->cdrom, 1, &msf, end_track, endmsf, &error) == FALSE) {
-			g_warning ("%s: %s", __FUNCTION__, error->message);
+			g_warning ("%s: %s", G_GNUC_FUNCTION, error->message);
 			g_error_free (error);
 
 			g_free (status);
@@ -130,7 +140,7 @@ play_cb (GtkButton *button,
 	switch (status->audio) {
 	case GNOME_CDROM_AUDIO_PLAY:
 		if (gnome_cdrom_pause (gcd->cdrom, &error) == FALSE) {
-			g_warning ("%s: %s", __FUNCTION__, error->message);
+			g_warning ("%s: %s", G_GNUC_FUNCTION, error->message);
 			g_error_free (error);
 
 			g_free (status);
@@ -159,7 +169,7 @@ play_cb (GtkButton *button,
 		}
 		if (gnome_cdrom_play (gcd->cdrom, status->track,
 				      &status->relative, end_track, endmsf, &error) == FALSE) {
-			g_warning ("%s: %s", __FUNCTION__, error->message);
+			g_warning ("%s: %s", G_GNUC_FUNCTION, error->message);
 			g_error_free (error);
 			
 			g_free (status);
@@ -189,7 +199,7 @@ play_cb (GtkButton *button,
 			endmsf = &msf;
 		}
 		if (gnome_cdrom_play (gcd->cdrom, 1, &msf, end_track, endmsf, &error) == FALSE) {
-			g_warning ("%s: %s", __FUNCTION__, error->message);
+			g_warning ("%s: %s", G_GNUC_FUNCTION, error->message);
 			g_error_free (error);
 
 			g_free (status);
@@ -228,7 +238,7 @@ stop_cb (GtkButton *button,
 	maybe_close_tray (gcd);
 
 	if (gnome_cdrom_stop (gcd->cdrom, &error) == FALSE) {
-		g_warning ("%s: %s", __FUNCTION__, error->message);
+		g_warning ("%s: %s", G_GNUC_FUNCTION, error->message);
 		g_error_free (error);
 	}
 
@@ -251,7 +261,7 @@ ffwd_timeout_cb (gpointer data)
 	GnomeCD *gcd = data;
 
 	if (gnome_cdrom_fast_forward (gcd->cdrom, &error) == FALSE) {
-		g_warning ("%s: %s", __FUNCTION__, error->message);
+		g_warning ("%s: %s", G_GNUC_FUNCTION, error->message);
 		g_error_free (error);
 		return FALSE;
 	}
@@ -310,7 +320,7 @@ next_cb (GtkButton *button,
 	maybe_close_tray (gcd);
 
 	if (gnome_cdrom_next (gcd->cdrom, &error) == FALSE) {
-		g_warning ("%s: %s", __FUNCTION__, error->message);
+		g_warning ("%s: %s", G_GNUC_FUNCTION, error->message);
 		g_error_free (error);
 	}
 
@@ -334,7 +344,7 @@ back_cb (GtkButton *button,
 	maybe_close_tray (gcd);
 
 	if (gnome_cdrom_back (gcd->cdrom, &error) == FALSE) {
-		g_warning ("%s: %s", __FUNCTION__, error->message);
+		g_warning ("%s: %s", G_GNUC_FUNCTION, error->message);
 		g_error_free (error);
 	}
 
@@ -356,7 +366,7 @@ rewind_timeout_cb (gpointer data)
 	GnomeCD *gcd = data;
 	
 	if (gnome_cdrom_rewind (gcd->cdrom, &error) == FALSE) {
-		g_warning ("%s: %s", __FUNCTION__, error->message);
+		g_warning ("%s: %s", G_GNUC_FUNCTION, error->message);
 		g_error_free (error);
 		return FALSE;
 	}
@@ -442,6 +452,7 @@ cd_status_changed_cb (GnomeCDRom *cdrom,
 	
 	switch (status->cd) {
 	case GNOME_CDROM_STATUS_OK:
+		gtk_widget_set_sensitive (gcd->trackeditor_b, TRUE);
 		if (status->audio == GNOME_CDROM_AUDIO_COMPLETE) {
 			if (cdrom->loopmode == GNOME_CDROM_LOOP) {
 				if (cdrom->playmode == GNOME_CDROM_WHOLE_CD) {
@@ -463,7 +474,7 @@ cd_status_changed_cb (GnomeCDRom *cdrom,
 					
 					if (gnome_cdrom_play (gcd->cdrom, start_track, &msf,
 							      end_track, &msf, &error) == FALSE) {
-						g_warning ("%s: %s", __FUNCTION__, error->message);
+						g_warning ("%s: %s", G_GNUC_FUNCTION, error->message);
 						g_error_free (error);
 						
 						g_free (status);
@@ -498,8 +509,8 @@ cd_status_changed_cb (GnomeCDRom *cdrom,
 		g_free (text);
 
 		if (gcd->disc_info == NULL) {
-			cd_display_set_line (CD_DISPLAY (gcd->display), CD_DISPLAY_LINE_ARTIST, "Unknown Artist");
-			cd_display_set_line (CD_DISPLAY (gcd->display), CD_DISPLAY_LINE_ALBUM, "Unknown Album");
+			cd_display_set_line (CD_DISPLAY (gcd->display), CD_DISPLAY_LINE_ARTIST, _("Unknown Artist"));
+			cd_display_set_line (CD_DISPLAY (gcd->display), CD_DISPLAY_LINE_ALBUM, _("Unknown Album"));
 			gnome_cd_set_window_title (gcd, NULL, NULL);
 		} else {
 			GnomeCDDiscInfo *info = gcd->disc_info;
@@ -513,7 +524,8 @@ cd_status_changed_cb (GnomeCDRom *cdrom,
 
 			if (status->audio == GNOME_CDROM_AUDIO_PLAY ||
 			    status->audio == GNOME_CDROM_AUDIO_PAUSE) {
-				gnome_cd_set_window_title (gcd, info->artist, info->tracknames[status->track - 1]);
+				gnome_cd_set_window_title (gcd, info->artist,
+							   info->track_info[status->track - 1]->name);
 			} else {
 				gnome_cd_set_window_title (gcd, info->artist, info->title);
 			}
@@ -522,33 +534,42 @@ cd_status_changed_cb (GnomeCDRom *cdrom,
 
 		
 	case GNOME_CDROM_STATUS_NO_DISC:
+		if (gcd->disc_info != NULL) {
+			cddb_free_disc_info (gcd->disc_info);
+			gcd->disc_info = NULL;
+		}
+		
+		gtk_widget_set_sensitive (gcd->trackeditor_b, FALSE);
 		cd_display_clear (CD_DISPLAY (gcd->display));
-		cd_display_set_line (CD_DISPLAY (gcd->display), CD_DISPLAY_LINE_TIME, "No disc");
+		cd_display_set_line (CD_DISPLAY (gcd->display), CD_DISPLAY_LINE_TIME, _("No disc"));
 		gnome_cd_set_window_title (gcd, NULL, NULL);
 		break;
 
 	case GNOME_CDROM_STATUS_TRAY_OPEN:
+		if (gcd->disc_info != NULL) {
+			cddb_free_disc_info (gcd->disc_info);
+			gcd->disc_info = NULL;
+		}
+
+		gtk_widget_set_sensitive (gcd->trackeditor_b, FALSE);
 		cd_display_clear (CD_DISPLAY (gcd->display));
-		cd_display_set_line (CD_DISPLAY (gcd->display), CD_DISPLAY_LINE_TIME, "Drive open");
+		cd_display_set_line (CD_DISPLAY (gcd->display), CD_DISPLAY_LINE_TIME, _("Drive open"));
 		gnome_cd_set_window_title (gcd, NULL, NULL);
 		break;
 
 	default:
+		if (gcd->disc_info != NULL) {
+			cddb_free_disc_info (gcd->disc_info);
+			gcd->disc_info = NULL;
+		}
+
+		gtk_widget_set_sensitive (gcd->trackeditor_b, FALSE);
 		cd_display_clear (CD_DISPLAY (gcd->display));
-		cd_display_set_line (CD_DISPLAY (gcd->display), CD_DISPLAY_LINE_TIME, "Drive Error");
+		cd_display_set_line (CD_DISPLAY (gcd->display), CD_DISPLAY_LINE_TIME, _("Drive Error"));
 		gnome_cd_set_window_title (gcd, NULL, NULL);
 		break;
 	}
 
-#ifdef DEBUG
-	g_print ("Status changed\n"
-		 "Device: %d\nAudio %d\n"
-		 "Track: %d (%d:%d:%d)\n"
-		 "--------------------\n",
-		 status->cd, status->audio,
-		 status->track, status->relative.minute,
-		 status->relative.second, status->relative.frame);
-#endif
 	if (gcd->last_status != NULL) {
 		g_free (gcd->last_status);
 	}
@@ -605,3 +626,87 @@ open_preferences (GtkWidget *widget,
 		gdk_window_raise (dialog->window);
 	}
 }
+
+#define CDDBSLAVE_TRACK_EDITOR_IID "OAFIID:GNOME_Media_CDDBSlave2_TrackEditor"
+
+static void
+restart_track_editor (gpointer data)
+{
+	CORBA_Environment ev;
+	
+	g_print ("Trackeditor died. Restarting\n");
+	CORBA_exception_init (&ev);
+	track_editor = bonobo_activation_activate_from_id (CDDBSLAVE_TRACK_EDITOR_IID, 0, NULL, &ev);
+	if (BONOBO_EX (&ev)) {
+		g_warning ("Could not reactivate track editor.\n%s", CORBA_exception_id (&ev));
+		track_editor = CORBA_OBJECT_NIL;
+		CORBA_exception_free (&ev);
+
+		return;
+	}
+
+	ORBit_small_listen_for_broken (track_editor, G_CALLBACK (restart_track_editor), &ev);
+	CORBA_exception_free (&ev);
+}
+
+void
+destroy_track_editor (void)
+{
+	if (track_editor != CORBA_OBJECT_NIL) {
+		ORBit_small_unlisten_for_broken (track_editor, G_CALLBACK (restart_track_editor));
+		bonobo_object_release_unref (track_editor, NULL);
+	}
+}
+
+void
+open_track_editor (GtkWidget *widget,
+		   GnomeCD *gcd)
+{
+	GnomeCDRomCDDBData *data;
+	CORBA_Environment ev;
+	char *discid;
+	
+	if (gnome_cdrom_get_cddb_data (gcd->cdrom, &data, NULL) == FALSE) {
+		g_print ("gnome_cdrom_get_cddb_data returned FALSE\n");
+		return;
+	}
+	
+	CORBA_exception_init (&ev);
+	if (track_editor == CORBA_OBJECT_NIL) {
+		track_editor = bonobo_activation_activate_from_id (CDDBSLAVE_TRACK_EDITOR_IID, 0, NULL, &ev);
+		if (BONOBO_EX (&ev)) {
+			/* FIXME: Should be an error dialog */
+			g_warning ("Could not activate track editor.\n%s",
+				   CORBA_exception_id (&ev));
+			CORBA_exception_free (&ev);
+			
+			return;
+		}
+		
+		if (track_editor == CORBA_OBJECT_NIL) {
+			/* FIXME: Should be an error dialog */
+			g_warning ("Could not start track editor.");
+			
+			return;
+		}
+
+		/* Listen for the trackeditor dying on us,
+		   then restart it */
+		ORBit_small_listen_for_broken (track_editor, G_CALLBACK (restart_track_editor), &ev);
+	}
+
+	if (data != NULL) {
+		discid = g_strdup_printf ("%08lx", data->discid);
+		GNOME_Media_CDDBTrackEditor_setDiscID (track_editor, discid, &ev);
+		g_free (discid);
+	}
+	
+	GNOME_Media_CDDBTrackEditor_showWindow (track_editor, &ev);
+	if (BONOBO_EX (&ev)) {
+		/* FIXME: Should be an error dialog */
+		CORBA_exception_free (&ev);
+		bonobo_object_release_unref (track_editor, NULL);
+		return;
+	}
+	CORBA_exception_free (&ev);
+}
