@@ -40,7 +40,7 @@ get_disc_info (GnomeCD *gcd,
 
 	info->title = cddb_slave_client_get_disc_title (slave, discid);
 	info->artist = cddb_slave_client_get_artist (slave, discid);
-	info->tracknames = cddb_slave_client_get_tracks (slave, discid);
+	info->track_info = cddb_slave_client_get_tracks (slave, discid);
 
 	gnome_cd_build_track_list_menu (gcd);
 }
@@ -91,6 +91,17 @@ cddb_listener_event_cb (BonoboListener *listener,
 	}
 }
 
+void
+cddb_free_disc_info (GnomeCDDiscInfo *info)
+{
+	g_free (info->discid);
+	g_free (info->title);
+	g_free (info->artist);
+
+	cddb_slave_client_free_track_info (info->track_info);
+	g_free (info);
+}
+
 static GnomeCDDiscInfo *
 cddb_make_disc_info (GnomeCDRomCDDBData *data)
 {
@@ -101,11 +112,18 @@ cddb_make_disc_info (GnomeCDRomCDDBData *data)
 	discinfo->title = NULL;
 	discinfo->artist = NULL;
 	discinfo->ntracks = data->ntrks;
-	discinfo->tracknames = NULL;
+	discinfo->track_info = NULL;
 
 	return discinfo;
 }
 
+void
+close_cddb_client (void)
+{
+	if (slave != NULL) {
+		g_object_unref (G_OBJECT (slave));
+	}
+}
 void
 cddb_get_query (GnomeCD *gcd)
 {
@@ -153,12 +171,14 @@ cddb_get_query (GnomeCD *gcd)
 	/* Remove the last space */
 	offsets[strlen (offsets) - 1] = 0;
 
-	slave = cddb_slave_client_new ();
-	listener = bonobo_listener_new (NULL, NULL);
-	g_signal_connect (G_OBJECT (listener), "event-notify",
-			  G_CALLBACK (cddb_listener_event_cb), gcd);
+	if (slave == NULL) {
+		slave = cddb_slave_client_new ();
+		listener = bonobo_listener_new (NULL, NULL);
+		g_signal_connect (G_OBJECT (listener), "event-notify",
+				  G_CALLBACK (cddb_listener_event_cb), gcd);
 
-	cddb_slave_client_add_listener (slave, listener);
+		cddb_slave_client_add_listener (slave, listener);
+	}
 
 	cddb_slave_client_query (slave, discid, data->ntrks, offsets, 
 				 data->nsecs, "GnomeCD", VERSION);
