@@ -19,6 +19,7 @@
 #include "cdrom.h"
 #include "callbacks.h"
 #include "display.h"
+#include "access/factory.h"
 
 #define DEFAULT_THEME "lcd"
 
@@ -94,18 +95,18 @@ gnome_cd_build_track_list_menu (GnomeCD *gcd)
 		gtk_option_menu_remove_menu (GTK_OPTION_MENU (gcd->tracks));
 	}
 
-	menu = gtk_menu_new ();
+	menu = GTK_MENU(gtk_menu_new ());
 	if (gcd->disc_info != NULL) {
 		int i;
 		for (i = 0; i < gcd->disc_info->ntracks; i++) {
 			char *title;
 
-			title = g_strdup_printf ("%d - %s", i + 1, gcd->disc_info->tracknames[i]);
+			title = g_strdup_printf ("%d - %s", i + 1, gcd->disc_info->track_info[i]->name);
 			item = gtk_menu_item_new_with_label (title);
 			g_free (title);
 			gtk_widget_show (item);
 
-			gtk_menu_append (menu, item);
+			gtk_menu_shell_append (GTK_MENU_SHELL(menu), item);
 		}
 	} else {
 		GnomeCDRomCDDBData *data;
@@ -127,7 +128,7 @@ gnome_cd_build_track_list_menu (GnomeCD *gcd)
 							
 							gtk_widget_show (item);
 							
-							gtk_menu_append (menu, item);
+							gtk_menu_shell_append ((GtkMenuShell*)(menu), item);
 						}
 						
 						g_free (data);
@@ -140,7 +141,7 @@ gnome_cd_build_track_list_menu (GnomeCD *gcd)
 	}
 
 	gtk_option_menu_set_menu (GTK_OPTION_MENU (gcd->tracks), GTK_WIDGET (menu));
-	gcd->menu = menu;
+	gcd->menu = GTK_WIDGET(menu);
 
 	g_signal_handlers_unblock_matched (G_OBJECT (gcd->tracks), G_SIGNAL_MATCH_FUNC, 0, 0, NULL,
 					   G_CALLBACK (skip_to_track), gcd);
@@ -181,7 +182,8 @@ make_button_from_file (GnomeCD *gcd,
 	GtkWidget *pixmap;
 	char *fullname;
 
-	fullname = gnome_pixmap_file (filename);
+	fullname = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_PIXMAP,
+		   filename, TRUE, NULL);
 	g_return_val_if_fail (fullname != NULL, NULL);
 
 	pixmap = gtk_image_new_from_file (fullname);
@@ -210,7 +212,8 @@ pixbuf_from_file (const char *filename)
 	GdkPixbuf *pixbuf;
 	char *fullname;
 
-	fullname = gnome_pixmap_file (filename);
+	fullname = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_PIXMAP, 
+                   filename, TRUE, NULL);
 	g_return_val_if_fail (fullname != NULL, NULL);
 
 	pixbuf = gdk_pixbuf_new_from_file (fullname, NULL);
@@ -246,6 +249,12 @@ window_destroy_cb (GtkWidget *window,
 		g_assert_not_reached ();
 		break;
 	}
+
+	/* Unref the cddb slave */
+	close_cddb_client ();
+
+	/* And the track editor */
+	destroy_track_editor ();
 	
 	g_object_unref (gcd->cdrom);
 	bonobo_main_quit ();
@@ -287,7 +296,9 @@ make_popup_menu (GnomeCD *gcd)
 			} else {
 				char *fullname;
 
-				fullname = gnome_pixmap_file (menuitems[i].icon);
+				fullname = gnome_program_locate_file (NULL, 
+				GNOME_FILE_DOMAIN_PIXMAP, menuitems[i].icon,
+                                TRUE, NULL);
 				if (fullname != NULL) {
 					image = gtk_image_new_from_file (fullname);
 				} else {
@@ -299,13 +310,13 @@ make_popup_menu (GnomeCD *gcd)
 
 			if (image != NULL) {
 				gtk_widget_show (image);
-				gtk_image_menu_item_set_image (item, image);
+				gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM(item), image);
 			}
 		}
 				
 		gtk_widget_show (item);
 
-		gtk_menu_append (GTK_MENU (menu), item);
+		gtk_menu_shell_append ((GtkMenuShell *)(menu), item);
 		if (menuitems[i].callback != NULL) {
 			g_signal_connect (G_OBJECT (item), "activate",
 					  G_CALLBACK (menuitems[i].callback), gcd);
@@ -329,7 +340,7 @@ init_player (void)
 	gcd = g_new0 (GnomeCD, 1);
 
 	gcd->not_ready = TRUE;
-	gcd->preferences = preferences_new (gcd);
+	gcd->preferences = (GnomeCDPreferences *)preferences_new (gcd);
 	if (gcd->preferences->device == NULL || gcd->preferences->device[0] == 0) {
 		GtkWidget *dialog;
 
@@ -346,7 +357,7 @@ init_player (void)
 		switch (gtk_dialog_run (GTK_DIALOG (dialog))) {
 		case 1:
 			gtk_widget_destroy (dialog);
-			dialog = preferences_dialog_show (gcd, TRUE);
+			dialog = GTK_WIDGET(preferences_dialog_show (gcd, TRUE));
 
 			/* Don't care what it returns */
 			gtk_dialog_run (GTK_DIALOG (dialog));
@@ -362,7 +373,7 @@ init_player (void)
 	gcd->cdrom = gnome_cdrom_new (gcd->preferences->device, GNOME_CDROM_UPDATE_CONTINOUS, &error);
 	if (gcd->cdrom == NULL) {
 		if (error != NULL) {
-			g_warning ("%s: %s", __FUNCTION__, error->message);
+			g_warning ("%s: %s", G_GNUC_FUNCTION, error->message);
 			g_error_free (error);
 		}
 	} else {
@@ -381,12 +392,12 @@ init_player (void)
 		gtk_dialog_add_buttons (GTK_DIALOG (dialog), GTK_STOCK_QUIT, GTK_RESPONSE_CLOSE,
 					_("Set device"), 1, NULL);
 		gtk_dialog_set_default_response (GTK_DIALOG (dialog), 1);
-		gtk_window_set_title (GTK_WINDOW (dialog), _("No CD device"));
+		gtk_window_set_title (GTK_WINDOW (dialog), _("Invalid CD device"));
 		
 		switch (gtk_dialog_run (GTK_DIALOG (dialog))) {
 		case 1:
 			gtk_widget_destroy (dialog);
-			dialog = preferences_dialog_show (gcd, TRUE);
+			dialog = GTK_WIDGET(preferences_dialog_show (gcd, TRUE));
 
 			/* Don't care what it returns */
 			gtk_dialog_run (GTK_DIALOG (dialog));
@@ -422,12 +433,12 @@ init_player (void)
 
 	/* Create app controls */
 	side_vbox = gtk_vbox_new (FALSE, 0);
-	button = make_button_from_stock (gcd, GTK_STOCK_PREFERENCES, NULL, _("Open track editor"), _("Track editor"));
+	button = make_button_from_stock (gcd, GTK_STOCK_INDEX, G_CALLBACK(open_track_editor), _("Open track editor"), _("Track editor"));
 	gtk_widget_set_sensitive (button, FALSE);
 	gtk_box_pack_start (GTK_BOX (side_vbox), button, FALSE, FALSE, 0);
 	gcd->trackeditor_b = button;
 
-	button = make_button_from_stock (gcd, GTK_STOCK_PROPERTIES, open_preferences, _("Preferences"), _("Preferences"));
+	button = make_button_from_stock (gcd, GTK_STOCK_PREFERENCES, G_CALLBACK(open_preferences), _("Preferences"), _("Preferences"));
 	gtk_box_pack_start (GTK_BOX (side_vbox), button, FALSE, FALSE, 0);
 	gcd->properties_b = button;
 
@@ -444,7 +455,7 @@ init_player (void)
 			  G_CALLBACK (playmode_changed_cb), gcd);
 
 	/* Theme needs to be loaded after the display is created */
-	gcd->theme = theme_load (gcd, gcd->preferences->theme_name);
+	gcd->theme = (GCDTheme *)theme_load (gcd, gcd->preferences->theme_name);
 	if (gcd->theme == NULL) {
 		g_error ("Could not create theme");
 	}
@@ -465,11 +476,11 @@ init_player (void)
 	
 	button_hbox = gtk_hbox_new (TRUE, 2);
 	
-	button = make_button_from_file (gcd, "gnome-cd/a-first.png", G_CALLBACK (back_cb), _("Previous track"), _("Previous"));
+  	button = make_button_from_file (gcd, "gnome-cd/media-prev.png", G_CALLBACK (back_cb), _("Previous track"), _("Previous"));
 	gtk_box_pack_start (GTK_BOX (button_hbox), button, TRUE, TRUE, 0);
 	gcd->back_b = button;
 
-	button = make_button_from_file (gcd, "gnome-cd/a-rwnd.png", NULL, _("Rewind"), _("Rewind"));
+	button = make_button_from_file (gcd, "gnome-cd/media-rew.png", NULL, _("Rewind"), _("Rewind"));
 	g_signal_connect (G_OBJECT (button), "button-press-event",
 			  G_CALLBACK (rewind_press_cb), gcd);
 	g_signal_connect (G_OBJECT (button), "button-release-event",
@@ -479,12 +490,14 @@ init_player (void)
 
 	/* Create the play and pause images, and ref them so they never
 	   get destroyed */
-	fullname = gnome_pixmap_file ("gnome-cd/a-play.png");
+	fullname = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_PIXMAP,
+                   "gnome-cd/media-play.png", TRUE, NULL);
 	gcd->play_image = gtk_image_new_from_file (fullname);
 	g_object_ref (gcd->play_image);
 	g_free (fullname);
 
-	fullname = gnome_pixmap_file ("gnome-cd/a-pause.png");
+	fullname = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_PIXMAP, 
+                   "gnome-cd/media-pause.png", TRUE, NULL);
 	gcd->pause_image = gtk_image_new_from_file (fullname);
 	gtk_widget_show (gcd->pause_image);
 	g_object_ref (gcd->pause_image);
@@ -495,11 +508,11 @@ init_player (void)
 	gcd->play_b = button;
 	gcd->current_image = gcd->play_image;
 	
-	button = make_button_from_file (gcd, "gnome-cd/a-stop.png", G_CALLBACK (stop_cb), _("Stop"), _("Stop"));
+	button = make_button_from_file (gcd, "gnome-cd/media-stop.png", G_CALLBACK (stop_cb), _("Stop"), _("Stop"));
 	gtk_box_pack_start (GTK_BOX (button_hbox), button, TRUE, TRUE, 0);
 	gcd->stop_b = button;
 
-	button = make_button_from_file (gcd, "gnome-cd/a-fwd.png", NULL, _("Fast forward"), _("Fast forward"));
+	button = make_button_from_file (gcd, "gnome-cd/media-fwd.png", NULL, _("Fast forward"), _("Fast forward"));
 	g_signal_connect (G_OBJECT (button), "button-press-event",
 			  G_CALLBACK (ffwd_press_cb), gcd);
 	g_signal_connect (G_OBJECT (button), "button-release-event",
@@ -507,7 +520,7 @@ init_player (void)
 	gtk_box_pack_start (GTK_BOX (button_hbox), button, TRUE, TRUE, 0);
 	gcd->ffwd_b = button;
 
-	button = make_button_from_file (gcd, "gnome-cd/a-last.png", G_CALLBACK (next_cb), _("Next track"), _("Next track"));
+	button = make_button_from_file (gcd, "gnome-cd/media-next.png", G_CALLBACK (next_cb), _("Next track"), _("Next track"));
 	gtk_box_pack_start (GTK_BOX (button_hbox), button, TRUE, TRUE, 0);
 	gcd->next_b = button;
 	
@@ -534,11 +547,39 @@ init_player (void)
 	return gcd;
 }
 
+static int 
+save_session(GnomeClient        *client,
+             gint                phase,
+             GnomeRestartStyle   save_style,
+             gint                shutdown,
+             GnomeInteractStyle  interact_style,
+             gint                fast,
+             gpointer            client_data) 
+{
+
+    gchar *argv[]= { NULL };
+
+    argv[0] = (gchar*) client_data;
+    gnome_client_set_clone_command (client, 1, argv);
+    gnome_client_set_restart_command (client, 1, argv);
+
+    return TRUE;
+}
+
+static gint client_die(GnomeClient *client,
+		       GnomeCD *gcd)
+{
+	gtk_widget_destroy (gcd->window);
+	gtk_main_quit ();
+}
+
+
 int 
 main (int argc,
       char *argv[])
 {
 	GnomeCD *gcd;
+	GnomeClient *client;
 
 	bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
@@ -547,12 +588,19 @@ main (int argc,
 	gnome_program_init ("Gnome-CD", VERSION, LIBGNOMEUI_MODULE, 
 			    argc, argv, NULL);
 	gconf_init (argc, argv, NULL);
+	client = gnome_master_client ();
+    	g_signal_connect (client, "save_yourself",
+                         G_CALLBACK (save_session), (gpointer) argv[0]);
+
 	gcd = init_player ();
 	if (gcd == NULL) {
 		g_error (_("Cannot create player"));
 		exit (0);
 	}
 
+	g_signal_connect (client, "die",
+			  G_CALLBACK (client_die), gcd);
+
 	/* Do the start up stuff */
 	if (gcd->preferences->start_close) {
 		gnome_cdrom_close_tray (gcd->cdrom, NULL);
@@ -576,6 +624,8 @@ main (int argc,
 	
 	gtk_widget_show (gcd->window);
 
+	setup_factory();
+
 	bonobo_main ();
 	return 0;
 }
