@@ -1,7 +1,7 @@
 /*
  * gnome-cd.h
  *
- * Copyright (C) 2001 Iain Holmes
+ * Copyright (C) 2001, 2002 Iain Holmes
  * Authors: Iain Holmes  <iain@ximian.com>
  */
 
@@ -13,8 +13,9 @@
 
 #include <pango/pango.h>
 
+#include <cddb-slave-client.h>
+
 #include "gnome-cd-type.h"
-#include "display.h"
 #include "preferences.h"
 #include "cdrom.h"
 
@@ -25,7 +26,7 @@ typedef struct _GnomeCDDiscInfo {
 	char *title;
 	char *artist;
 	int ntracks;
-	char **tracknames;
+	CDDBSlaveClientTrackInfo **track_info;
 } GnomeCDDiscInfo;
 
 typedef struct _GnomeCDText {
