@@ -16,7 +16,6 @@
 #include <linux/cdrom.h>
 #include <errno.h>
 
-#include "gnome-cd.h"
 #include "linux-cdrom.h"
 
 static GnomeCDRomClass *parent_class = NULL;
@@ -332,6 +331,13 @@ linux_cdrom_next (GnomeCDRom *cdrom,
 		return FALSE;
 	}
 
+	if (status->cd != GNOME_CDROM_STATUS_OK) {
+		linux_cdrom_close (lcd);
+		g_free (status);
+		
+		return TRUE;
+	}
+
 	track = status->track + 1;
 	g_free (status);
 	if (track > lcd->priv->number_tracks) {
@@ -380,6 +386,13 @@ linux_cdrom_ffwd (GnomeCDRom *cdrom,
 		return FALSE;
 	}
 
+	if (status->cd != GNOME_CDROM_STATUS_OK) {
+		linux_cdrom_close (lcd);
+		g_free (status);
+		
+		return TRUE;
+	}
+
 	msf = &status->absolute;
 	/* Convert MSF to frames to do calculations on it */
 	frames = msf_to_frames (msf);
@@ -691,6 +704,13 @@ linux_cdrom_rewind (GnomeCDRom *cdrom,
 		return FALSE;
 	}
 
+	if (status->cd != GNOME_CDROM_STATUS_OK) {
+		linux_cdrom_close (lcd);
+		g_free (status);
+		
+		return TRUE;
+	}
+	
 	msf = &status->absolute;
 
 	frames = msf_to_frames (msf);
@@ -750,6 +770,13 @@ linux_cdrom_back (GnomeCDRom *cdrom,
 		return FALSE;
 	}
 
+	if (status->cd != GNOME_CDROM_STATUS_OK) {
+		linux_cdrom_close (lcd);
+		g_free (status);
+		
+		return TRUE;
+	}
+
 	/* If we're > 0:00 on the track go back to the start of it, 
 	   otherwise go to the previous track */
 	if (status->relative.minute != 0 || status->relative.second != 0) {
@@ -990,7 +1017,6 @@ linux_cdrom_get_cddb_data (GnomeCDRom *cdrom,
 
 	for (i = priv->track0 - 1; i < priv->track1; i++) {
 		(*data)->offsets[i] = msf_to_frames (&priv->track_info[i].address);
-		g_print ("%d: %u\n", i, msf_to_frames (&priv->track_info[i].address));
 	}
 
 	linux_cdrom_close (lcd);
@@ -1089,12 +1115,12 @@ update_cd (gpointer data)
 
 	/* Do an update */
 	if (linux_cdrom_get_status (GNOME_CDROM (lcd), &status, &error) == FALSE) {
-		g_warning ("%s: %s", __FUNCTION__, error->message);
-
+		g_warning ("%s: %s", G_GNUC_FUNCTION, error->message);
+		
 		g_error_free (error);
 		return TRUE;
 	}
-
+	
 	if (priv->recent_status == NULL) {
 		priv->recent_status = status;
 		
@@ -1114,7 +1140,7 @@ update_cd (gpointer data)
 			    status->cd == GNOME_CDROM_STATUS_OK) {
 				linux_cdrom_update_cd (lcd);
 			}
-
+			
 			g_free (priv->recent_status);
 			priv->recent_status = status;
 
@@ -1166,8 +1192,8 @@ gnome_cdrom_new (const char *cdrom_device,
 	priv->update = update;
 
 	if (linux_cdrom_open (cdrom, error) == FALSE) {
-		linux_cdrom_close (cdrom);
 	}
+	linux_cdrom_close (cdrom);
 	
 	/* Force an update so that a status will always exist */
 	update_cd (cdrom);
