@@ -24,6 +24,10 @@
 #endif
 
 #include <gnome.h>
+
+
+#include <gconf/gconf-client.h>
+
 #include <sys/types.h>
 #include <sys/wait.h>
 #include <sys/stat.h>
@@ -48,7 +52,7 @@
 #define DONTSAVE 1
 #define CANCEL 2
 
-const gchar* maintopic = N_("GNOME Soundrecorder:");
+const gchar* maintopic = N_("Sound Recorder:");
 
 const gchar* temp_filename_record = "untitled.raw";
 const gchar* temp_filename_play = "untitled.wav";
@@ -111,9 +115,26 @@ on_play_activate_cb (GtkWidget* widget, gpointer data)
 
 	PlayEng.pid = fork ();
 	if (PlayEng.pid == 0) {
+#if 0
+		int i;
+		
 		/* Play file */
+		if (playrepeat == TRUE &&
+		    playrepeatforever == FALSE) {
+			for (i = 0; i < playxtimes; i++) {
+				g_print ("%d of %d: Playing %s\n", i, playxtimes, active_file);
+				play_sound (active_file);
+			}
+		} else if (playrepeat == TRUE &&
+			   playrepeatforever == TRUE) {
+			while (1) {
+				play_sound (active_file);
+			}
+		} else if (playrepeat == FALSE) {
+			play_sound (active_file);
+		}
+#endif
 		play_sound (active_file);
-
 		_exit (0);
 	}
 	else if (PlayEng.pid == -1)
@@ -133,7 +154,7 @@ on_stop_activate_cb (GtkWidget* widget, gpointer data)
 	gchar* temp_string1 = NULL;
 	gchar* temp_string2 = NULL;
 
-	gnome_appbar_pop (GNOME_APPBAR (grecord_widgets.appbar));
+	gnome_appbar_clear_stack (GNOME_APPBAR (grecord_widgets.appbar));
 
 	if (RecEng.is_running) {
 		temp_string1 = g_strconcat ("-r ", samplerate, NULL);
@@ -160,8 +181,10 @@ on_stop_activate_cb (GtkWidget* widget, gpointer data)
 	if (RecEng.is_running) {
 		gchar* command;
 		gchar* temp_string;
-		gchar* tfile1 = g_concat_dir_and_file (temp_dir, temp_filename_record);
-		gchar* tfile2 = g_concat_dir_and_file (temp_dir, temp_filename_play);
+		gchar* tfile1 = g_build_filename (temp_dir, 
+						  temp_filename_record, NULL);
+		gchar* tfile2 = g_build_filename (temp_dir, 
+						  temp_filename_play, NULL);
 		
 		command = g_strconcat (sox_command, " ", temp_string1, " ", temp_string2, " ",
 				       "-w ", "-s ", tfile1, " ",
@@ -174,7 +197,8 @@ on_stop_activate_cb (GtkWidget* widget, gpointer data)
 
 		run_command (command, _("Converting file..."));
 
-		temp_string = g_concat_dir_and_file (temp_dir, temp_filename_play);
+		temp_string = g_build_filename (temp_dir, 
+						temp_filename_play, NULL);
 
 		g_free (temp_string);
 		g_free (command);
@@ -192,15 +216,15 @@ on_new_activate_cb (GtkWidget* widget, gpointer data)
 	gint choice;
 	gchar* string = NULL;
 	gchar* temp_string = NULL;
-	gchar* file1 = g_concat_dir_and_file (temp_dir, temp_filename_record);
-	gchar* file2 = g_concat_dir_and_file (temp_dir, temp_filename_play);
-	gchar* file3 = g_concat_dir_and_file (temp_dir, temp_filename_backup);
+	gchar* file1 = g_build_filename (temp_dir, temp_filename_record, NULL);
+	gchar* file2 = g_build_filename (temp_dir, temp_filename_play, NULL);
+	gchar* file3 = g_build_filename (temp_dir, temp_filename_backup, NULL);
 
 	if (PlayEng.is_running || RecEng.is_running || convert_is_running)
 		on_stop_activate_cb (widget, data);
 
 	if (file_changed) {
-		choice = save_dont_or_cancel ();
+		choice = save_dont_or_cancel (_("Cancel"));
 		if (choice == SAVE) {
 			save_dialog ();
 			return;
@@ -220,10 +244,13 @@ on_new_activate_cb (GtkWidget* widget, gpointer data)
 
 	default_file = TRUE;
 
-	active_file = g_concat_dir_and_file (temp_dir, temp_filename_play);
+	if (active_file != NULL) {
+		g_free (active_file);
+	}
+	active_file = g_build_filename (temp_dir, temp_filename_play, NULL);
 	file_changed = FALSE;
 
-	set_min_sec_time (get_play_time (active_file), TRUE);
+	set_min_sec_time (get_play_time (active_file));
 
 	grecord_set_sensitive_nofile ();
 
@@ -273,7 +300,7 @@ on_open_activate_cb (GtkWidget* widget, gpointer data)
 	}
 
 	if (file_changed) {
-		choice = save_dont_or_cancel ();
+		choice = save_dont_or_cancel (_("Cancel open"));
 		if (choice == SAVE) {
 			save_dialog ();
 			return;
@@ -284,20 +311,16 @@ on_open_activate_cb (GtkWidget* widget, gpointer data)
 
 	filesel = gtk_file_selection_new (_("Select a sound file"));
 
-	gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (filesel)->ok_button),
-			    "clicked", 
-			    GTK_SIGNAL_FUNC (store_filename),
-			    filesel);
+	g_signal_connect ((GTK_FILE_SELECTION (filesel)->ok_button),
+			  "clicked", G_CALLBACK (store_filename),
+			  filesel);
 
-	gtk_signal_connect (GTK_OBJECT (filesel),
-			    "destroy",
-			    GTK_SIGNAL_FUNC (gtk_widget_destroyed),
-			    &filesel);
+	g_signal_connect (filesel, "destroy", G_CALLBACK (gtk_widget_destroyed),
+			  &filesel);
 
-	gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION (filesel)->cancel_button),
-				   "clicked", 
-				   GTK_SIGNAL_FUNC (gtk_widget_destroy),
-				   (gpointer) filesel);
+	g_signal_connect_swapped ((GTK_FILE_SELECTION (filesel)->cancel_button),
+				  "clicked", G_CALLBACK (gtk_widget_destroy),
+				  (gpointer) filesel);
 
 	gtk_widget_show (filesel);
 }
@@ -333,7 +356,7 @@ on_exit_activate_cb (GtkWidget* widget, gpointer data)
 		on_stop_activate_cb (widget, data);
 
 	if (file_changed) {
-		choice = save_dont_or_cancel ();
+		choice = save_dont_or_cancel (_("Don't quit"));
 		if (choice == SAVE) {
 			save_dialog ();
 			return;
@@ -343,9 +366,9 @@ on_exit_activate_cb (GtkWidget* widget, gpointer data)
 	}
 
 	/* User didn't want to save; copy the backup file to the changed file (active file) */
-	tfile = g_concat_dir_and_file (temp_dir, temp_filename_backup);
+	tfile = g_build_filename (temp_dir, temp_filename_backup, NULL);
 	
-	if (g_file_exists (tfile)) {
+	if (g_file_test (tfile, G_FILE_TEST_EXISTS)) {
 		command = g_strconcat ("cp -f ", tfile, " ", active_file, NULL);
 		system (command);
 		g_free (command);
@@ -356,10 +379,10 @@ on_exit_activate_cb (GtkWidget* widget, gpointer data)
 	remove (tfile);
 	g_free (tfile);
 
-	tfile = g_concat_dir_and_file (temp_dir, temp_filename_record);
+	tfile = g_build_filename (temp_dir, temp_filename_record, NULL);
 	remove (tfile);
 	g_free (tfile);
-	tfile = g_concat_dir_and_file (temp_dir, temp_filename_play);
+	tfile = g_build_filename (temp_dir, temp_filename_play, NULL);
 	remove (tfile);
 	g_free (tfile);
 }
@@ -379,10 +402,8 @@ on_preferences_activate_cb (GtkWidget* widget, gpointer data)
 	}
 	props = create_grecord_propertybox ();
 	
-	gtk_signal_connect (GTK_OBJECT (props),
-			    "destroy",
-			    GTK_SIGNAL_FUNC (gtk_widget_destroyed),
-			    &props);
+	g_signal_connect (props, "destroy", G_CALLBACK (gtk_widget_destroyed),
+			  &props);
 	gtk_widget_show (props);
 }
 
@@ -392,30 +413,45 @@ on_about_activate_cb (GtkWidget* widget, gpointer data)
 	static GtkWidget* about = NULL;
 
 	if (about) {
-		if (about->window == NULL)
-			return;
-		
-		gdk_window_show (about->window);
-		gdk_window_raise (about->window);
+		gtk_window_present (GTK_WINDOW (about));
 		return;
 	}
 	
 	about = create_about ();
 
-	gtk_signal_connect (GTK_OBJECT (about),
-			    "destroy",
-			    GTK_SIGNAL_FUNC (gtk_widget_destroyed),
-			    &about);
+	g_signal_connect (G_OBJECT (about),
+			  "destroy",
+			  G_CALLBACK (gtk_widget_destroyed),
+			  &about);
 
 	gtk_widget_show (about);
 }
 
+/* These need to bring up an error dialog */
 void
 on_runmixer_activate_cb (GtkWidget* widget, gpointer data)
 {
-	gchar* temp_string = g_strconcat (mixer_command, " &", NULL);
-	system (temp_string);
-	g_free (temp_string);
+	char *mixer_path;
+	char *argv[2] = {NULL, NULL};
+	GError *error = NULL;
+	gboolean ret;
+
+	/* Open the mixer */
+	mixer_path = g_find_program_in_path ("gnome-volume-control");
+	if (mixer_path == NULL) {
+		g_warning (_("Gnome Volume Control is not installed in the path"));
+		return;
+	}
+	
+	argv[0] = mixer_path;
+	ret = g_spawn_async (NULL, argv, NULL, 0, NULL, NULL, NULL, &error);
+	if (ret == FALSE) {
+		g_warning (_("There was an error starting %s: %s"),
+			   mixer_path, error->message);
+		g_error_free (error);
+	}
+
+	g_free (mixer_path);
 }
 
 void 
@@ -431,7 +467,7 @@ on_decrease_speed_activate_cb (GtkWidget* widget, gpointer data)
 void
 on_add_echo_activate_cb (GtkWidget* widget, gpointer data)
 {
-	if (!g_file_exists (active_file))
+	if (!g_file_test (active_file, G_FILE_TEST_EXISTS))
 	    return;
 
 	file_changed = TRUE;
@@ -441,6 +477,7 @@ on_add_echo_activate_cb (GtkWidget* widget, gpointer data)
 void
 on_show_time_activate_cb (GtkWidget* widget, gpointer data)
 {
+	GConfClient *client;
 	GtkCheckMenuItem* item = GTK_CHECK_MENU_ITEM (widget);
 	gboolean active = item->active;
 
@@ -455,15 +492,15 @@ on_show_time_activate_cb (GtkWidget* widget, gpointer data)
 		gtk_widget_hide (grecord_widgets.timemin_label);
 	}
 
-	show_time = active;
-
-	gnome_config_set_bool ("/grecord/GUI Options/showtime", show_time);
-	gnome_config_sync ();
+	client = gconf_client_get_default ();
+	gconf_client_set_bool (client, "/apps/gnome-sound-recorder/show-time", active, NULL);
+	g_object_unref (G_OBJECT (client));
 }
 
 void
 on_show_soundinfo_activate_cb (GtkWidget* widget, gpointer data)
 {
+	GConfClient *client;
 	GtkCheckMenuItem* item = GTK_CHECK_MENU_ITEM (widget);
 	gboolean active = item->active;
 
@@ -478,10 +515,9 @@ on_show_soundinfo_activate_cb (GtkWidget* widget, gpointer data)
 		gtk_widget_hide (grecord_widgets.nr_of_channels_label);
 	}
 
-	show_soundinfo = active;
-
-	gnome_config_set_bool ("/grecord/GUI Options/showsoundinfo", show_soundinfo);
-	gnome_config_sync ();
+	client = gconf_client_get_default ();
+	gconf_client_set_bool (client, "/apps/gnome-sound-recorder/show-sound-info", active, NULL);
+	g_object_unref (G_OBJECT (client));
 }
 
 void
@@ -497,9 +533,10 @@ on_redo_activate_cb (GtkWidget* widget, gpointer data)
 void
 on_undoall_activate_cb (GtkWidget* widget, gpointer data)
 {
-	gchar* temp_string = g_concat_dir_and_file (temp_dir, temp_filename_backup);
+	gchar* temp_string = g_build_filename (temp_dir, 
+					       temp_filename_backup, NULL);
 
-	if (g_file_exists (temp_string)) {
+	if (g_file_test (temp_string, G_FILE_TEST_EXISTS)) {
 		gchar* t = g_strconcat ("cp -f ", temp_string, " ", active_file, NULL);
 		run_command (t, _("Undoing all changes..."));
 		g_free (t);
@@ -530,15 +567,18 @@ record_sound (void)
 	FILE *target = NULL;
 
 	/* Open the file for writing */
-	tfile = g_concat_dir_and_file (temp_dir, temp_filename_record);
+	tfile = g_build_filename (temp_dir, temp_filename_record, NULL);
 	target = fopen (tfile, "w");
 	g_free (tfile);
 
 	/* Set up bits, channels etc after the preferences */
-	if (esd_channels == 1)
+	if (channels == 1) {
 		esd_channels = ESD_MONO;
-	if (audioformat != 0)
+	}
+	
+	if (audioformat != 0) {
 		bits = ESD_BITS8;
+	}
 
 	rate = atoi (samplerate);
 
@@ -568,51 +608,56 @@ void
 store_filename (GtkFileSelection* selector, gpointer file_selector)
 {
 	GtkWidget* mess;
-	gchar* string = NULL;
-	gchar* temp_string = NULL;
+	char* string = NULL;
+	char* temp_string = NULL;
 	AFfilehandle filename;
 	gint in_audioformat, in_channels, in_rate, in_width;
 	struct stat file;
-	
-	gchar* tempfile = gtk_file_selection_get_filename (GTK_FILE_SELECTION (file_selector));
+	const char* tempfile;
+
+	tempfile = gtk_file_selection_get_filename (GTK_FILE_SELECTION (file_selector));
 
 	if (!stat(tempfile, &file)) {
 		if (S_ISDIR (file.st_mode)) {
-			gchar* show_mess = g_strdup_printf (_("'%s' is a directory.\nPlease select a soundfile to be opened."), tempfile);
-			mess = gnome_message_box_new (show_mess,
-						      GNOME_MESSAGE_BOX_ERROR,
-						      GNOME_STOCK_BUTTON_OK,
-					      NULL);
-			g_free (show_mess);
-			gtk_window_set_modal (GTK_WINDOW (mess), TRUE);
+			mess = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL,
+						       GTK_MESSAGE_ERROR,
+						       GTK_BUTTONS_OK,
+						       _("'%s is a directory.\nPlease select a sound file to be opened."),
+						       tempfile);
+			g_signal_connect (G_OBJECT (mess), "response",
+					  G_CALLBACK (gtk_widget_destroy), NULL);
+
 			gtk_widget_show (mess);
 			return;
 		}
-	}
-	else if (errno == ENOENT) {
-		gchar* show_mess = g_strdup_printf (_("File '%s' doesn't exist.\nPlease select a existing soundfile to be opened."), tempfile);
-		mess = gnome_message_box_new (show_mess,
-					      GNOME_MESSAGE_BOX_ERROR,
-					      GNOME_STOCK_BUTTON_OK,
-					      NULL);
-		g_free (show_mess);
-		gtk_window_set_modal (GTK_WINDOW (mess), TRUE);
+	} else if (errno == ENOENT) {
+		mess = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL,
+					    GTK_MESSAGE_ERROR,
+					    GTK_BUTTONS_OK,
+					    _("File '%s' doesn't exist.\nPlease select an existing sound file to be opened."),
+					    tempfile);
+		g_signal_connect (G_OBJECT (mess), "response",
+				  G_CALLBACK (gtk_widget_destroy), NULL);
+
 		gtk_widget_show (mess);
 		return;
 	}
 
 	if (!soundfile_supported (tempfile)) {
-		gchar* show_mess = g_strdup_printf (_("File '%s' isn't a valid soundfile."), tempfile);
-		mess = gnome_message_box_new (show_mess,
-					      GNOME_MESSAGE_BOX_ERROR,
-					      GNOME_STOCK_BUTTON_OK,
-					      NULL);
-		g_free (show_mess);
-		gtk_window_set_modal (GTK_WINDOW (mess), TRUE);
+		mess = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL,
+					       GTK_MESSAGE_ERROR,
+					       GTK_BUTTONS_OK,
+					       _("File '%s isn't a valid sound file."),
+					       tempfile);
+		g_signal_connect (G_OBJECT (mess), "response",
+				  G_CALLBACK (gtk_widget_destroy), NULL);
 		gtk_widget_show (mess);
 		return;
 	}
 
+	if (active_file != NULL) {
+		g_free (active_file);
+	}
 	active_file = g_strdup (tempfile);
 	file_changed = FALSE;
 	gtk_widget_destroy (GTK_WIDGET (file_selector));
@@ -629,7 +674,7 @@ store_filename (GtkFileSelection* selector, gpointer file_selector)
 	afGetSampleFormat (filename, AF_DEFAULT_TRACK, &in_audioformat, &in_width);
 
 	/* Update mainwindow with the new values and set topic */
-	set_min_sec_time (get_play_time (active_file), TRUE);
+	set_min_sec_time (get_play_time (active_file));
 
 	samplerate = g_strdup_printf ("%d", in_rate);
 	if (in_channels == 2)
@@ -662,7 +707,6 @@ store_filename (GtkFileSelection* selector, gpointer file_selector)
 void
 save_filename (GtkFileSelection* selector, gpointer file_selector)
 {
-	gchar* temp_string;
 	gchar* new_file;
 	struct stat file;
 
@@ -671,79 +715,116 @@ save_filename (GtkFileSelection* selector, gpointer file_selector)
 	/* Check if the file already exists */
 	if (!stat(new_file, &file)) {
 		GtkWidget* mess;
-		gchar* show_mess;
-		gint choice;
 
 		if (S_ISDIR (file.st_mode)) {
-			show_mess = g_strdup_printf (_("'%s' is a directory.\nPlease enter another filename."), new_file);
-			mess = gnome_message_box_new (show_mess,
-						      GNOME_MESSAGE_BOX_ERROR,
-						      GNOME_STOCK_BUTTON_OK,
-						      NULL);
-			g_free (show_mess);
-			gtk_window_set_modal (GTK_WINDOW (mess), TRUE);
+			mess = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL,
+						       GTK_MESSAGE_ERROR,
+						       GTK_BUTTONS_OK,
+						       _("'%s' is a directory.\nPlease enter another filename."),
+						       new_file);
+			g_signal_connect (G_OBJECT (mess), "response",
+					  G_CALLBACK (gtk_widget_destroy), NULL);
+			
 			gtk_widget_show (mess);
+			g_free (new_file);
 			return;
 		}
-		show_mess = g_strdup_printf (_("File '%s' already exists.\nDo you want to overwrite it?"), new_file);
-		mess = gnome_message_box_new (show_mess,
-					      GNOME_MESSAGE_BOX_WARNING,
-					      GNOME_STOCK_BUTTON_YES,
-					      GNOME_STOCK_BUTTON_NO,
-					      GNOME_STOCK_BUTTON_CANCEL,
-					      NULL);
-		choice = gnome_dialog_run (GNOME_DIALOG (mess));
-		g_free (show_mess);
-		if (choice == 2 || choice == 1)
+		mess = gtk_message_dialog_new (NULL, 0,
+					       GTK_MESSAGE_WARNING,
+					       GTK_BUTTONS_NONE,
+					       _("<b>File '%s' already exists.</b>\nDo you want to overwrite it?"),
+					       new_file);
+		gtk_dialog_add_buttons (GTK_DIALOG (mess),
+					_("Cancel save"), 0,
+					_("Overwrite"), 1,
+					NULL);
+
+		/* Bad hack usage #2 */
+		gtk_label_set_use_markup (GTK_LABEL (GTK_MESSAGE_DIALOG (mess)->label), TRUE);
+		gtk_label_set_line_wrap (GTK_LABEL (GTK_MESSAGE_DIALOG (mess)->label), FALSE);
+		gtk_label_set_justify (GTK_LABEL (GTK_MESSAGE_DIALOG (mess)->label), GTK_JUSTIFY_LEFT);
+
+		switch (gtk_dialog_run (GTK_DIALOG (mess))) {
+		case 0:
+			gtk_widget_destroy (mess);
+			g_free (new_file);
 			return;
+
+		default:
+			gtk_widget_destroy (mess);
+			break;
+		}
 	}
 
 	/* Check if the soundfile is supported */
 	if (!save_sound_file (new_file)) {
-		GtkWidget* mess = gnome_message_box_new (_("Error saving sound file"),
-							 GNOME_MESSAGE_BOX_ERROR,
-							 GNOME_STOCK_BUTTON_OK,
-							 NULL);
-		gtk_window_set_modal (GTK_WINDOW (mess), TRUE);
+		GtkWidget *mess;
+
+		mess = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL,
+					       GTK_MESSAGE_ERROR,
+					       GTK_BUTTONS_OK,
+					       _("Error saving '%s'"),
+					       new_file);
+		g_signal_connect (G_OBJECT (mess), "response",
+				  G_CALLBACK (gtk_widget_destroy), NULL);
+		
 		gtk_widget_show (mess);
+		
 		g_free (new_file);
 		return;
 	}
 
-	if (active_file[1] == '/') {
-		gchar* tempstring;
-		gchar* string = g_strdup ((char *) strrchr (active_file, '/'));
-		string[1] = ' ';
-		tempstring = g_strconcat (_(maintopic), string, NULL);
-		gtk_window_set_title (GTK_WINDOW (grecord_widgets.grecord_window), tempstring);
-		g_free (tempstring);
-		g_free (string);
+	if (active_file != NULL) {
+		g_free (active_file);
 	}
-
-	active_file = g_strdup (new_file);
-	g_free (new_file);
+	active_file = new_file;
 	
 	gtk_widget_destroy (GTK_WIDGET (file_selector));
 
-	temp_string = g_strconcat ((maintopic), active_file, NULL);
-	gtk_window_set_title (GTK_WINDOW (grecord_widgets.grecord_window), temp_string);
-	g_free (temp_string);
-
+	set_window_title (active_file);
 	file_changed = FALSE;
 	file_selector = NULL;
 }
 
 gint
-save_dont_or_cancel (void)
+save_dont_or_cancel (const char *quit_text)
 {
-	GtkWidget* mess = gnome_message_box_new (_("File not saved. Do you want to save it?"),
-						 GNOME_MESSAGE_BOX_QUESTION,
-						 GNOME_STOCK_BUTTON_YES,
-						 GNOME_STOCK_BUTTON_NO,
-						 GNOME_STOCK_BUTTON_CANCEL,
-						 NULL);
+	int result;
+	char *filename;
+	char *title;
+	GtkWidget *mess;
 
-	return (gnome_dialog_run (GNOME_DIALOG (mess)));
+	if (active_file == NULL) {
+		filename = g_strdup ("untitled.wav");
+	} else {
+		filename = g_path_get_basename (active_file);
+	}
+	
+	mess = gtk_message_dialog_new (grecord_widgets.grecord_window, 0,
+				       GTK_MESSAGE_WARNING,
+				       GTK_BUTTONS_NONE,
+				       _("<b>Save the changes to %s?</b>\nIf you don't save, these changes will be discarded."),
+				       filename);
+	
+	gtk_dialog_add_buttons (GTK_DIALOG (mess),
+				_("Don't save"), DONTSAVE,
+				quit_text, CANCEL,
+				_("Save"), SAVE, NULL);
+	gtk_dialog_set_default_response (GTK_DIALOG (mess), 0);
+
+	title = g_strdup_printf (_("Save %s?"), filename);
+	gtk_window_set_title (GTK_WINDOW (mess), title);
+	g_free (title);
+	g_free (filename);
+	
+	/* This is a bad hack, it's marked private, but tough */
+	gtk_label_set_use_markup (GTK_LABEL (GTK_MESSAGE_DIALOG (mess)->label), TRUE);
+	gtk_label_set_line_wrap (GTK_LABEL (GTK_MESSAGE_DIALOG (mess)->label), FALSE);
+	gtk_label_set_justify (GTK_LABEL (GTK_MESSAGE_DIALOG (mess)->label), GTK_JUSTIFY_LEFT);
+	
+	result = gtk_dialog_run (GTK_DIALOG (mess));
+	gtk_widget_destroy (mess);
+	return result;
 }
 
 gboolean
@@ -751,7 +832,8 @@ save_sound_file (const gchar* filename)
 {
 	/* Check if the file is default (if it's been recorded) */
 	if (default_file) {
-		gchar* tfile = g_concat_dir_and_file (temp_dir, temp_filename_play);
+		gchar* tfile = g_build_filename (temp_dir, 
+						 temp_filename_play, NULL);
 		gchar* command = g_strconcat ("cp -f ", tfile, " ", filename, NULL);
 		
 		/* Save the file */
@@ -783,22 +865,20 @@ UpdateStatusbarPlay (gboolean begin)
 
 		length = get_play_time (active_file);
 		temp = 1000 / length;
-	}
-	else {
+	} else {
 		if (waitpid (PlayEng.pid, NULL, WNOHANG | WUNTRACED)) {
 			if (playrepeat && playrepeatforever) {
 				on_play_activate_cb (NULL, NULL);
 				return FALSE;
 			}
 			else if (playrepeat && !playrepeatforever) {
-				if (repeat_counter < playxtimes) {
+				if (repeat_counter < playxtimes - 1) {
 					on_play_activate_cb (NULL, NULL);
 					repeat_counter++;
 					return FALSE;
 				}
 				repeat_counter = 0;
 			}
-			
 			counter = 0;
 			PlayEng.is_running = FALSE;
 			grecord_set_sensitive_file ();
@@ -811,7 +891,7 @@ UpdateStatusbarPlay (gboolean begin)
 
 	countersec++;
 
-	set_min_sec_time (countersec, FALSE);
+	set_min_sec_time (countersec);
 
 	gtk_range_set_adjustment (GTK_RANGE (grecord_widgets.Statusbar), GTK_ADJUSTMENT (gtk_adjustment_new (counter, 0, 1000, 1, 1, 0)));
 /*  	gtk_range_slider_update (GTK_RANGE (grecord_widgets.Statusbar)); */
@@ -860,13 +940,14 @@ UpdateStatusbarRecord (gboolean begin)
 
 	countersec++;
 	
-	set_min_sec_time (countersec, FALSE);
+	set_min_sec_time (countersec);
 	
 	if (popup_warn_mess) {
 		struct stat fileinfo;
 		static gint maxfilesize = -1;
 		static gboolean show_message = TRUE;
-		gchar* filename = g_concat_dir_and_file (temp_dir, temp_filename_record);
+		gchar* filename = g_build_filename (temp_dir, 
+						    temp_filename_record, NULL);
 		
 		if (maxfilesize == -1)
 			maxfilesize = popup_warn_mess_v;
@@ -876,14 +957,16 @@ UpdateStatusbarRecord (gboolean begin)
 		g_free (filename);
 		
 		if (fileinfo.st_size >= (maxfilesize * 1000000) && show_message) {
-			gchar* message = g_strdup_printf (N_("The size of the current sample is more than %i Mb!"),
-							  (int) (fileinfo.st_size / 1000000) /* In MB */);
-			GtkWidget* mess = gnome_message_box_new (_(message),
-								 GNOME_MESSAGE_BOX_WARNING,
-								 GNOME_STOCK_BUTTON_OK,
-								 NULL);
+			GtkWidget *mess;
+			
+			mess = gtk_message_dialog_new (grecord_widgets.grecord_window,
+						       0, GTK_MESSAGE_WARNING,
+						       GTK_BUTTONS_OK,
+						       _("The size of the current sample is more than\n%i Mb!"),
+						       (int) (fileinfo.st_size / 1000000));
+			g_signal_connect (G_OBJECT (mess), "response",
+					  G_CALLBACK (gtk_widget_destroy), NULL);
 			gtk_widget_show (mess);
-			g_free (message);
 			show_message = FALSE;
 	        }			
 	}
@@ -891,7 +974,8 @@ UpdateStatusbarRecord (gboolean begin)
 	if (stop_record) {
 		struct stat fileinfo;
 		static gint maxfilesize = -1;
-		gchar* filename = g_concat_dir_and_file (temp_dir, temp_filename_record);
+		gchar* filename = g_build_filename (temp_dir, 
+						    temp_filename_record, NULL);
 
 		if (maxfilesize == -1)
 			maxfilesize = stop_record_v;
@@ -922,13 +1006,16 @@ void
 save_dialog (void)
 {
 	GtkWidget* filesel = NULL;
-	gchar* temp_file = g_concat_dir_and_file (temp_dir, temp_filename_play);
+	gchar* temp_file = g_build_filename (temp_dir, 
+					     temp_filename_play, NULL);
 
 	filesel = gtk_file_selection_new (_("Save sound file"));
 
 	if (!g_strcasecmp (active_file, temp_file)) {
 		gchar* home_dir = g_strdup (getenv ("HOME"));
-		gchar* default_file = g_concat_dir_and_file (home_dir, _(temp_filename_play));
+		gchar* default_file = g_build_filename (home_dir, 
+							_(temp_filename_play),
+							NULL);
 
 		gtk_file_selection_set_filename (GTK_FILE_SELECTION (filesel), default_file);
 
@@ -938,15 +1025,12 @@ save_dialog (void)
 	else
 		gtk_file_selection_set_filename (GTK_FILE_SELECTION (filesel), active_file);
 		
-	gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (filesel)->ok_button),
-			    "clicked", 
-			    GTK_SIGNAL_FUNC (save_filename),
-			    filesel);
+	g_signal_connect ((GTK_FILE_SELECTION (filesel)->ok_button),
+			  "clicked", G_CALLBACK (save_filename), filesel);
 
-	gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION (filesel)->cancel_button),
-				   "clicked", 
-				   GTK_SIGNAL_FUNC (gtk_widget_destroy),
-				   (gpointer) filesel);
+	g_signal_connect_swapped ((GTK_FILE_SELECTION (filesel)->cancel_button),
+				  "clicked", G_CALLBACK (gtk_widget_destroy),
+				  (gpointer) filesel);
 
 	gtk_window_set_modal (GTK_WINDOW (filesel), TRUE);
 	
@@ -959,7 +1043,8 @@ void
 is_file_default (void)
 {
 	gchar* temp_string;
-	temp_string = g_concat_dir_and_file (temp_dir, temp_filename_play);
+	temp_string = g_build_filename (temp_dir, temp_filename_play, NULL);
+
 	if (!g_strcasecmp (temp_string, active_file))
 	        default_file = TRUE;
 	else
@@ -1044,10 +1129,10 @@ check_if_loading_finished (gint pid)
 	if (waitpid (pid, NULL, WNOHANG | WUNTRACED)) {
 		
 		/* Show the playtime of the file */
-		set_min_sec_time (get_play_time (active_file), TRUE);
+		set_min_sec_time (get_play_time (active_file));
 		
 		/* Remove the comment from the appbar, because we're finished */
-		gnome_appbar_pop (GNOME_APPBAR (grecord_widgets.appbar));
+		gnome_appbar_clear_stack (GNOME_APPBAR (grecord_widgets.appbar));
 
 		/* Make widgets sensitive again */
 		grecord_set_sensitive_file ();
@@ -1067,7 +1152,7 @@ soundfile_supported (const gchar* filename)
 	gint soundtype = afGetFileFormat (filetype, NULL);
 
 	/* Check if the file exists */
-	if (!g_file_exists (filename))
+	if (!g_file_test (filename, G_FILE_TEST_EXISTS))
 		return FALSE;
 
 	/* Check if the file is a valid soundfile */
@@ -1091,12 +1176,12 @@ check_if_sounddevice_ready ()
 		GtkWidget* mess;
 
 		esd_audio_close ();
-
-		mess = gnome_message_box_new (_("Sounddevice not ready! Please check that there isn't\nanother program running that's using the sounddevice."),
-					      GNOME_MESSAGE_BOX_ERROR,
-					      GNOME_STOCK_BUTTON_OK,
-					      NULL);
-		gnome_dialog_run (GNOME_DIALOG (mess));
+		mess = gtk_message_dialog_new (NULL, 0,
+					       GTK_MESSAGE_ERROR,
+					       GTK_BUTTONS_OK,
+					       _("The sound device is not ready. Please check that there "
+						 "isn't\nanother program running that is using the device."));
+		gtk_dialog_run (GTK_DIALOG (mess));
 		return FALSE;
 	}
 
